% Statistical Model for the Between-subjects Variability of Internal
% Carotid Flow Based on 103 Elderly Subjects
%
% Usage:
%
%  load FlowVariabilityModel_ICA_LassilaEtAl18;
%  tt = linspace(0,4,100); % Plot four heartbeats at 1 Hz sampling
%  SimulateCBF( Model, gender, height, plotSignals, tt );
%
% where
%
%  gender        - 1 for women, 0 for men
%  height        - should be provided in meters
%  plotSignals   - controls whether the generated signals will be plotted
%  tt (optional) - determines the temporal sampling of the waveform
%
% This program generates virtual waveforms representing internal carotid
% waveform variability. The model was trained in a cohort of 103 middle-
% aged and elderly subjects as part of  adementia study conducted within 
% the VPH-SHARE@IT FP7 project. It is intended only for research purposes. 
%
% Please cite the reference below when using this model.
%
% Reference: Lassila T, Sarrami-Foroushani A, Hejazi S, Frangi AF. 
% Population-Specific Modelling of Between/Within-Subject Flow Variability 
% in the Carotid Arteries of the Elderly, preprint, 2018.
%
% Copyright license: CC-BY-4.0

function [CBF_L,CBF_R] = SimulateCBF(Model, gender, height, plotSignals, tt)

% Use model trained for women or men as appropriate
if (gender == 1)
    CC = Model.CCF;
else
    CC = Model.CCM;
end

if (nargin < 5)
    % Default time points
    tt = linspace(0,1,100);
else
    max_heartbeats = ceil(max(tt));
    tt_default = linspace(0,max_heartbeats,100);
end

landmarks = [ 1 8 10 12 16 18 20 22 24 26 28 32 34 38 42 46 60 95 ];

% Draw random waveform from the between-subjects variability model
W = randn(2*length(landmarks),1);
signal = CC' * W;
CBF_L = signal(1:length(landmarks))';
CBF_R = signal(length(landmarks)+1:2*length(landmarks))';

% Extend sampling of virtual signals to the time
METHOD = 'linear';

if (gender == 1)
    ICA_L_MEAN = Model.ICA_L_MEAN_FEMALE;
    ICA_R_MEAN = Model.ICA_R_MEAN_FEMALE;
else
    ICA_L_MEAN = Model.ICA_L_MEAN_MALE;
    ICA_R_MEAN = Model.ICA_R_MEAN_MALE;
end

% Extrapolate from landmarks to entire sampling interval
if (nargin < 5)
    CBF_L = interp1(tt(landmarks), CBF_L + ICA_L_MEAN(landmarks), tt, METHOD,'extrap');
    CBF_R = interp1(tt(landmarks), CBF_R + ICA_R_MEAN(landmarks), tt, METHOD,'extrap');
else
    tt2 = [];
    for i = 0 : max_heartbeats-1
        tt2 = [tt2 landmarks/100+i];
    end
    CBF_L = interp1(tt2, repmat(CBF_L + ICA_L_MEAN(landmarks),1,max_heartbeats), tt, METHOD, 'extrap');
    CBF_R = interp1(tt2, repmat(CBF_R + ICA_R_MEAN(landmarks),1,max_heartbeats), tt, METHOD, 'extrap');
end

% Convert Log-transformed values to real values (in units of ml/min)
if (gender == 1)
    CBF_L = exp(glmval(Model.BFL, log(height), 'identity') + CBF_L + Model.SIGMAFL * randn);
    CBF_R = exp(glmval(Model.BFR, log(height), 'identity') + CBF_R + Model.SIGMAFR * randn);
else
    CBF_L = exp(glmval(Model.BML, log(height), 'identity') + CBF_L + Model.SIGMAML*randn);
    CBF_R = exp(glmval(Model.BMR, log(height), 'identity') + CBF_R + Model.SIGMAMR*randn);
end

% Plot the signals (optional)
if (plotSignals)
    figure(1)
    
    subplot(2,1,1)
    plot(tt, CBF_L)
    grid on
    ylabel('ICA-L flow rate [ml/min]')
    
    subplot(2,1,2)
    plot(tt, CBF_R)
    grid on
    xlabel('Normalised time')
    ylabel('ICA-R flow rate [ml/min]')
end
